"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  anatomy: () => anatomy,
  connect: () => connect,
  itemProps: () => itemProps,
  machine: () => machine,
  props: () => props,
  splitItemProps: () => splitItemProps,
  splitProps: () => splitProps
});
module.exports = __toCommonJS(src_exports);

// src/file-upload.anatomy.ts
var import_anatomy = require("@zag-js/anatomy");
var anatomy = (0, import_anatomy.createAnatomy)("file-upload").parts(
  "root",
  "dropzone",
  "item",
  "itemDeleteTrigger",
  "itemGroup",
  "itemName",
  "itemPreview",
  "itemPreviewImage",
  "itemSizeText",
  "label",
  "trigger"
);
var parts = anatomy.build();

// src/file-upload.connect.ts
var import_dom_query2 = require("@zag-js/dom-query");
var import_i18n_utils = require("@zag-js/i18n-utils");
var import_types = require("@zag-js/types");

// src/file-upload.dom.ts
var import_dom_query = require("@zag-js/dom-query");
var dom = (0, import_dom_query.createScope)({
  getRootId: (ctx) => ctx.ids?.root ?? `file:${ctx.id}`,
  getDropzoneId: (ctx) => ctx.ids?.dropzone ?? `file:${ctx.id}:dropzone`,
  getHiddenInputId: (ctx) => ctx.ids?.hiddenInput ?? `file:${ctx.id}:input`,
  getTriggerId: (ctx) => ctx.ids?.trigger ?? `file:${ctx.id}:trigger`,
  getLabelId: (ctx) => ctx.ids?.label ?? `file:${ctx.id}:label`,
  getItemId: (ctx, id) => ctx.ids?.item?.(id) ?? `file:${ctx.id}:item:${id}`,
  getItemNameId: (ctx, id) => ctx.ids?.itemName?.(id) ?? `file:${ctx.id}:item-name:${id}`,
  getItemSizeTextId: (ctx, id) => ctx.ids?.itemSizeText?.(id) ?? `file:${ctx.id}:item-size:${id}`,
  getItemPreviewId: (ctx, id) => ctx.ids?.itemPreview?.(id) ?? `file:${ctx.id}:item-preview:${id}`,
  getHiddenInputEl: (ctx) => dom.getById(ctx, dom.getHiddenInputId(ctx)),
  getDropzoneEl: (ctx) => dom.getById(ctx, dom.getDropzoneId(ctx))
});

// src/file-upload.utils.ts
var import_file_utils = require("@zag-js/file-utils");
function isEventWithFiles(event) {
  if (!event.dataTransfer)
    return !!event.target && "files" in event.target;
  return event.dataTransfer.types.some((type) => {
    return type === "Files" || type === "application/x-moz-file";
  });
}
function isFilesWithinRange(ctx, incomingCount) {
  if (!ctx.multiple && incomingCount > 1)
    return false;
  if (!ctx.multiple && incomingCount + ctx.acceptedFiles.length === 2)
    return true;
  if (incomingCount + ctx.acceptedFiles.length > ctx.maxFiles)
    return false;
  return true;
}
function getFilesFromEvent(ctx, files) {
  const acceptedFiles = [];
  const rejectedFiles = [];
  files.forEach((file) => {
    const [accepted, acceptError] = (0, import_file_utils.isValidFileType)(file, ctx.acceptAttr);
    const [sizeMatch, sizeError] = (0, import_file_utils.isValidFileSize)(file, ctx.minFileSize, ctx.maxFileSize);
    const validateErrors = ctx.validate?.(file);
    const valid = validateErrors ? validateErrors.length === 0 : true;
    if (accepted && sizeMatch && valid) {
      acceptedFiles.push(file);
    } else {
      const errors = [acceptError, sizeError];
      if (!valid)
        errors.push(...validateErrors ?? []);
      rejectedFiles.push({ file, errors: errors.filter(Boolean) });
    }
  });
  if (!isFilesWithinRange(ctx, acceptedFiles.length)) {
    acceptedFiles.forEach((file) => {
      rejectedFiles.push({ file, errors: ["TOO_MANY_FILES"] });
    });
    acceptedFiles.splice(0);
  }
  return {
    acceptedFiles,
    rejectedFiles
  };
}

// src/file-upload.connect.ts
function connect(state, send, normalize) {
  const disabled = state.context.disabled;
  const allowDrop = state.context.allowDrop;
  const translations = state.context.translations;
  const dragging = state.matches("dragging");
  const focused = state.matches("focused") && !disabled;
  return {
    dragging,
    focused,
    openFilePicker() {
      send("OPEN");
    },
    deleteFile(file) {
      send({ type: "FILE.DELETE", file });
    },
    acceptedFiles: state.context.acceptedFiles,
    rejectedFiles: state.context.rejectedFiles,
    setFiles(files) {
      const count = files.length;
      send({ type: "FILES.SET", files, count });
    },
    clearFiles() {
      send({ type: "FILES.CLEAR" });
    },
    getFileSize(file) {
      return (0, import_i18n_utils.formatBytes)(file.size, state.context.locale);
    },
    createFileUrl(file, cb) {
      const win = dom.getWin(state.context);
      const url = win.URL.createObjectURL(file);
      cb(url);
      return () => win.URL.revokeObjectURL(url);
    },
    rootProps: normalize.element({
      ...parts.root.attrs,
      dir: state.context.dir,
      id: dom.getRootId(state.context),
      "data-disabled": (0, import_dom_query2.dataAttr)(disabled),
      "data-dragging": (0, import_dom_query2.dataAttr)(dragging)
    }),
    dropzoneProps: normalize.element({
      ...parts.dropzone.attrs,
      dir: state.context.dir,
      id: dom.getDropzoneId(state.context),
      tabIndex: disabled ? void 0 : 0,
      "aria-disabled": disabled,
      "aria-invalid": state.context.invalid,
      "data-invalid": (0, import_dom_query2.dataAttr)(state.context.invalid),
      "data-disabled": (0, import_dom_query2.dataAttr)(disabled),
      "data-dragging": (0, import_dom_query2.dataAttr)(dragging),
      onKeyDown(event) {
        if (event.defaultPrevented)
          return;
        const evt = event.nativeEvent || event;
        if (!(0, import_dom_query2.isSelfTarget)(evt))
          return;
        if (event.key !== "Enter" && event.key !== " ")
          return;
        send({ type: "DROPZONE.CLICK", src: "keydown" });
      },
      onClick(event) {
        const isLabel = event.currentTarget.localName === "label";
        if (isLabel)
          event.preventDefault();
        send("DROPZONE.CLICK");
      },
      onDragOver(event) {
        if (!allowDrop)
          return;
        event.preventDefault();
        event.stopPropagation();
        try {
          event.dataTransfer.dropEffect = "copy";
        } catch {
        }
        const hasFiles = isEventWithFiles(event);
        if (!hasFiles)
          return;
        const count = event.dataTransfer.items.length;
        send({ type: "DROPZONE.DRAG_OVER", count });
      },
      onDragLeave(event) {
        if (!allowDrop || disabled)
          return;
        if ((0, import_dom_query2.contains)(event.currentTarget, event.relatedTarget))
          return;
        send({ type: "DROPZONE.DRAG_LEAVE" });
      },
      onDrop(event) {
        if (allowDrop) {
          event.preventDefault();
          event.stopPropagation();
        }
        const hasFiles = isEventWithFiles(event);
        if (disabled || !hasFiles)
          return;
        send({ type: "DROPZONE.DROP", files: Array.from(event.dataTransfer.files) });
      },
      onFocus() {
        send("DROPZONE.FOCUS");
      },
      onBlur() {
        send("DROPZONE.BLUR");
      }
    }),
    triggerProps: normalize.button({
      ...parts.trigger.attrs,
      dir: state.context.dir,
      id: dom.getTriggerId(state.context),
      disabled,
      "data-disabled": (0, import_dom_query2.dataAttr)(disabled),
      type: "button",
      onClick(event) {
        if (disabled)
          return;
        if ((0, import_dom_query2.contains)(dom.getDropzoneEl(state.context), event.currentTarget)) {
          event.stopPropagation();
        }
        send("OPEN");
      }
    }),
    hiddenInputProps: normalize.input({
      id: dom.getHiddenInputId(state.context),
      tabIndex: -1,
      disabled,
      type: "file",
      capture: state.context.capture,
      name: state.context.name,
      accept: state.context.acceptAttr,
      webkitdirectory: state.context.capture ? "" : void 0,
      multiple: state.context.multiple || state.context.maxFiles > 1,
      onClick(event) {
        event.stopPropagation();
        event.currentTarget.value = "";
      },
      onChange(event) {
        if (disabled)
          return;
        const { files } = event.currentTarget;
        send({ type: "FILES.SET", files: files ? Array.from(files) : [] });
      },
      style: import_dom_query2.visuallyHiddenStyle
    }),
    itemGroupProps: normalize.element({
      ...parts.itemGroup.attrs,
      dir: state.context.dir,
      "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
    }),
    getItemProps(props2) {
      const { file } = props2;
      return normalize.element({
        ...parts.item.attrs,
        dir: state.context.dir,
        id: dom.getItemId(state.context, file.name),
        "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
      });
    },
    getItemNameProps(props2) {
      const { file } = props2;
      return normalize.element({
        ...parts.itemName.attrs,
        dir: state.context.dir,
        id: dom.getItemNameId(state.context, file.name),
        "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
      });
    },
    getItemSizeTextProps(props2) {
      const { file } = props2;
      return normalize.element({
        ...parts.itemSizeText.attrs,
        dir: state.context.dir,
        id: dom.getItemSizeTextId(state.context, file.name),
        "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
      });
    },
    getItemPreviewProps(props2) {
      const { file } = props2;
      return normalize.element({
        ...parts.itemPreview.attrs,
        dir: state.context.dir,
        id: dom.getItemPreviewId(state.context, file.name),
        "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
      });
    },
    getItemPreviewImageProps(props2) {
      const { file, url } = props2;
      const isImage = file.type.startsWith("image/");
      if (!isImage) {
        throw new Error("Preview Image is only supported for image files");
      }
      return normalize.img({
        ...parts.itemPreviewImage.attrs,
        alt: translations.itemPreview(file),
        src: url,
        "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
      });
    },
    getItemDeleteTriggerProps(props2) {
      const { file } = props2;
      return normalize.button({
        ...parts.itemDeleteTrigger.attrs,
        dir: state.context.dir,
        type: "button",
        disabled,
        "data-disabled": (0, import_dom_query2.dataAttr)(disabled),
        "aria-label": translations.deleteFile(file),
        onClick() {
          if (disabled)
            return;
          send({ type: "FILE.DELETE", file });
        }
      });
    },
    labelProps: normalize.label({
      ...parts.label.attrs,
      dir: state.context.dir,
      id: dom.getLabelId(state.context),
      htmlFor: dom.getHiddenInputId(state.context),
      "data-disabled": (0, import_dom_query2.dataAttr)(disabled)
    })
  };
}

// src/file-upload.machine.ts
var import_core = require("@zag-js/core");
var import_dom_query3 = require("@zag-js/dom-query");
var import_file_utils2 = require("@zag-js/file-utils");
var import_utils = require("@zag-js/utils");
var { not } = import_core.guards;
function machine(userContext) {
  const ctx = (0, import_utils.compact)(userContext);
  return (0, import_core.createMachine)(
    {
      id: "fileupload",
      initial: "idle",
      context: {
        minFileSize: 0,
        maxFileSize: Infinity,
        maxFiles: 1,
        allowDrop: true,
        ...ctx,
        acceptedFiles: (0, import_core.ref)([]),
        rejectedFiles: (0, import_core.ref)([]),
        invalid: false,
        translations: {
          itemPreview: (file) => `preview of ${file.name}`,
          deleteFile: (file) => `delete file ${file.name}`,
          ...ctx.translations
        }
      },
      computed: {
        acceptAttr: (ctx2) => (0, import_file_utils2.getAcceptAttrString)(ctx2.accept),
        multiple: (ctx2) => ctx2.maxFiles > 1
      },
      on: {
        "FILES.SET": {
          actions: ["setFilesFromEvent"]
        },
        "FILE.DELETE": {
          actions: ["removeFile"]
        },
        "FILES.CLEAR": {
          actions: ["clearFiles"]
        }
      },
      states: {
        idle: {
          on: {
            OPEN: {
              actions: ["openFilePicker"]
            },
            "DROPZONE.CLICK": {
              actions: ["openFilePicker"]
            },
            "DROPZONE.FOCUS": "focused",
            "DROPZONE.DRAG_OVER": [
              {
                guard: not("isWithinRange"),
                target: "dragging",
                actions: ["setInvalid"]
              },
              { target: "dragging" }
            ]
          }
        },
        focused: {
          on: {
            "DROPZONE.BLUR": "idle",
            OPEN: {
              actions: ["openFilePicker"]
            },
            "DROPZONE.CLICK": {
              actions: ["openFilePicker"]
            },
            "DROPZONE.DRAG_OVER": [
              {
                guard: not("isWithinRange"),
                target: "dragging",
                actions: ["setInvalid"]
              },
              { target: "dragging" }
            ]
          }
        },
        dragging: {
          on: {
            "DROPZONE.DROP": {
              target: "idle",
              actions: ["clearInvalid", "setFilesFromEvent"]
            },
            "DROPZONE.DRAG_LEAVE": {
              target: "idle",
              actions: ["clearInvalid"]
            }
          }
        }
      }
    },
    {
      guards: {
        isWithinRange: (ctx2, evt) => isFilesWithinRange(ctx2, evt.count)
      },
      actions: {
        openFilePicker(ctx2) {
          (0, import_dom_query3.raf)(() => {
            dom.getHiddenInputEl(ctx2)?.click();
          });
        },
        setInvalid(ctx2) {
          ctx2.invalid = true;
        },
        clearInvalid(ctx2) {
          ctx2.invalid = false;
        },
        setFilesFromEvent(ctx2, evt) {
          const result = getFilesFromEvent(ctx2, evt.files);
          const { acceptedFiles, rejectedFiles } = result;
          if (ctx2.multiple) {
            const files = (0, import_core.ref)([...ctx2.acceptedFiles, ...acceptedFiles]);
            set.files(ctx2, files, rejectedFiles);
            return;
          }
          if (acceptedFiles.length) {
            const files = (0, import_core.ref)([acceptedFiles[0]]);
            set.files(ctx2, files, rejectedFiles);
          } else if (rejectedFiles.length) {
            set.files(ctx2, [], rejectedFiles);
          }
        },
        removeFile(ctx2, evt) {
          const nextFiles = ctx2.acceptedFiles.filter((file) => file !== evt.file);
          set.files(ctx2, nextFiles);
        },
        clearFiles(ctx2) {
          set.files(ctx2, []);
        }
      },
      compareFns: {
        acceptedFiles: (a, b) => a.length === b.length && a.every((file, i) => (0, import_file_utils2.isFileEqual)(file, b[i]))
      }
    }
  );
}
var invoke = {
  change: (ctx) => {
    ctx.onFileChange?.({
      acceptedFiles: ctx.acceptedFiles,
      rejectedFiles: ctx.rejectedFiles
    });
  },
  accept: (ctx) => {
    ctx.onFileAccept?.({ files: ctx.acceptedFiles });
  },
  reject: (ctx) => {
    ctx.onFileReject?.({ files: ctx.rejectedFiles });
  }
};
var set = {
  files: (ctx, acceptedFiles, rejectedFiles) => {
    ctx.acceptedFiles = (0, import_core.ref)(acceptedFiles);
    invoke.accept(ctx);
    if (rejectedFiles) {
      ctx.rejectedFiles = (0, import_core.ref)(rejectedFiles);
      invoke.reject(ctx);
    }
    invoke.change(ctx);
  }
};

// src/file-upload.props.ts
var import_types2 = require("@zag-js/types");
var import_utils2 = require("@zag-js/utils");
var props = (0, import_types2.createProps)()([
  "accept",
  "allowDrop",
  "capture",
  "dir",
  "directory",
  "disabled",
  "getRootNode",
  "id",
  "ids",
  "locale",
  "maxFiles",
  "maxFileSize",
  "minFileSize",
  "name",
  "onFileAccept",
  "onFileReject",
  "onFileChange",
  "translations",
  "validate"
]);
var splitProps = (0, import_utils2.createSplitProps)(props);
var itemProps = (0, import_types2.createProps)()(["file"]);
var splitItemProps = (0, import_utils2.createSplitProps)(itemProps);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  anatomy,
  connect,
  itemProps,
  machine,
  props,
  splitItemProps,
  splitProps
});
//# sourceMappingURL=index.js.map